# Take the matches and mismatches output files, and calulate the SNC thresholds
# Sl and Cl. 
# In the mismatch list, for each mismatch, choose the base change that occur most frequently (ie: has the highest number
# of indpendent reads)
# Threshold of Sl and Cl are from user input
# Sl = # of independent reads supporting a base change at position S / max # of reads at position S
# Cl = total # of all reads suppoting a base change at position S / # of all # reads that pass through position S
# Written by: Wendy Lee
# Written on: July 31st, 2008
# Last modified: November 17th, 2008
import sys
from time import strftime
from commoncode import *

print '%s: version 1.1' % sys.argv[0]

try:
	import psyco
	psyco.full()
except:
	print "psyco is not running"
	pass

def getMatchDict(matchfile):
        infile = file(matchfile, 'r')
        matchDict = {} 
        for line in infile:
                x = line.split();
                start = int(x[1])
                stop = int(x[2])
                chrom = x[0]
                try:
                        matchDict[chrom].append((start,stop))
                except:
                        matchDict[chrom] = [(start, stop)]
        return matchDict

def checkBetween(matchList, pos_in):
        c = 0
        for (start, stop) in matchList:
                if start <= pos_in <= stop:
                        c += 1
        return c

def getMismatchDict(mismatchfile):
        infile = file(mismatchfile, 'r')
        mismatchDict = {} 
        for x in infile:
                more = x.split(",") 
                chrom_in = more[0] 
                alotmore = more[1].split("[")
                pos_in = int(alotmore[0])
                counts = more[2].split("|")
                uniqCnt = alotmore[1][1:] 
                totalCnt = counts[0]
                basechanges = x.split("{")
                totbasechanges = basechanges[2].split("}")[0]
                basechangess = basechanges[1].split("}")
                basechange = basechangess[0].split(",")
                highest = ["",0]
                highTotDict = {} 
                highTot = ["",0]
                for k in totbasechanges.split(","):
                    kk = ""
                    kk = k.split(":")
                    bs = kk[0].split()[0][1:4] 
                    bsCnt = int(kk[1].split()[0])
                    highTotDict[bs] = bsCnt
                for x in basechange:
                        y = x.split(":")
                        yy = y[0].split()
                        base = yy[0][1:4]
                        if int(y[1]) > highest[1]:
                            highest = [base, int(y[1])]
                            highTot = [base, int(highTotDict[base])]
                         
                try:
                        mismatchDict[chrom_in].append((pos_in, [uniqCnt, totalCnt, highest, highTot]))
                except:
                        mismatchDict[chrom_in] = [(pos_in, [uniqCnt, totalCnt, highest, highTot])]
        return mismatchDict

#Main program 
if len(sys.argv) < 7:
        print 'usage: python2.5 %s matchesfile mismatchfile samplerdsfile sl_threshold cl_threshold outfile [-cache pages]' % sys.argv[0]
        sys.exit(1)

outStr = "" 
matchfile = sys.argv[1]
mismatchfile = sys.argv[2]
hitfile = sys.argv[3]
sl_threshold = float(sys.argv[4])
cl_threshold = float(sys.argv[5])
outfile = sys.argv[6]
cachePages = 0
doCache = False
if '-cache' in sys.argv:
    cachePages = int(sys.argv[sys.argv.index('-cache') + 1])
if cachePages > 0:
    doCache = True
hitRDS = readDataset(hitfile, verbose = True, cache=doCache)
if cachePages > 2000:
    hitRDS.setDBcache(cachePages)
readLength = hitRDS.getReadSize() 
print 'getting mismatch dict'
mismatchdict = getMismatchDict(mismatchfile)
print 'getting match dict'
matchdict = getMatchDict(matchfile)
print 'getting candidate SNPs'
filename  = open(outfile, 'w')
#outStr += "Sl\t" + "Cl\t" + "chrom" + "\t" + "mis pos" + "\t\t" + "match" + "\t" + "uniq_mis"+ "\t" + "tot_mis" + "\t" + "base_chg" + "\n"  
for chrom_in in mismatchdict:
        for x in mismatchdict[chrom_in]:
                pos_in = int(x[0])
                uniqCnt = str(x[1][0]) # all mismatches with unique start
                totalCnt = str(x[1][1]) # all mismatches
                highestCnt = str(x[1][2][1]) # highest mismatch with unique start
                highestBs = x[1][2][0] # highest mismatch base
                highTotCnt = str(x[1][3][1]) # all highest mismatch 
                highTotBs = x[1][3][0] # mismatch base of the highest of all mismatches
                Sl = "%1.*f" % (2, float(highestCnt)/float(readLength))
                Cl = float(0)
                if float(Sl) > sl_threshold/readLength:
                        matchCnt = checkBetween(matchdict[chrom_in], pos_in)
                        Cl = "%1.*f" % (2, float(highTotCnt)/(float(highTotCnt)+float(matchCnt)))
                # output the number of matches overlapping a given position
                        #print Sl, Cl, highTotCnt, str(float(highTotCnt)+float(matchCnt))
                if float(Cl)>cl_threshold:
                        outline = str(Sl) + "\t" + str(Cl) + "\t" + chrom_in + "\t" + str(pos_in) + "\t" + str(matchCnt) + "\t" + highestCnt + "\t\t" + highTotCnt + "\t" + highTotBs 
                        print outline
                        filename.write(outline + "\n") 
filename.close()

try:
        logfile = open("snp.log")
except IOError:
        logfile = open("snp.log", 'w')
else:
        logfile = open("snp.log", 'a')
now = strftime("%Y-%m-%d %H:%M:%S")

outStr = ""

outS = str(now) + ": getSNPs.py outputfile: " + outfile + "\n"
logfile.writelines(outS)
logfile.close()
