#!/bin/bash
#
# getSNPs.sh
#
# Usages: $ERANGEPATH/getSNPs.sh mouse rdsfile label rmaskdbfile dbsnpfile sl_threshold cl_threshold rpkmfile cachepages
# Example: /getSNPs.sh mouse /woldlab/trog/sdc/alim/24T4spike_10212/24T4spike.rds 24Tspike /woldlab/trog/data1/wlee/db/rmask.db /woldlab/trog/data1/wlee/db/dbSNP128.db 5 0.75 ~/proj/c2c12rna24R/c2c12rna.24R.final.rpkm 5000000 

# set ERANGEPATH to the absolute or relative path to ERANGE, if it's not in the environment

if [ -z "$ERANGEPATH" ]
then
    ERANGEPATH='../commoncode'
fi

echo $0': version 1.1'

if [ $# -lt 8 ]; then
    echo 'getSNPs.sh genome rdsfile label rmaskdbfile dbsnpfile sl_threshold cl_threshold rpkmfile [cachepages]'
    exit
fi
cachepages=""
if [ $# -eq 9 ]; then
	cachepages="-cache "$9
fi

# get all mismatches from rds
python2.5 $ERANGEPATH/getAllMismatches.py $2 $3.mismatches.txt $cachepages

# get all matches from rds
python2.5 $ERANGEPATH/getMatches.py $2 $3.matches.txt $cachepages

# get all SNPs using the matches and mismatches info
python2.5 $ERANGEPATH/getSNPs.py $3.matches.txt $3.mismatches.txt $2 $6 $7 $3.snps.txt $cachepages

# get SNPs in non-repeat regions only
python2.5 $ERANGEPATH/chkSNPrmask.py $4 $3.snps.txt $3.nr_snps.txt $cachepages

# Check to see if sncs are found in dbSNP
# if dbSNP128.db is not built yet, build it by running buildsnpdb.py - build snp database using the dbSNP database file downloaded from UCSC
# usage: python2.5 buildsnpdb.py snpdbdir snpdbname
# the database flat file must be in the snpdbdir directory
# To build dbSNP database file, run the following command 
# python2.5 buildsnpdb.py snp128.txt dbSNP128

# get dbSNP info for SNPs that are found in the dbSNP database
python2.5 $ERANGEPATH/chksnp.py $5 $3.nr_snps.txt $3.nr_dbsnp.txt $cachepages

# get gene info for the snps found in dbSNP
python2.5 $ERANGEPATH/getSNPGeneInfo.py $1 $3.nr_dbsnp.txt $8 $3.nr_dbsnp_geneinfo.txt $cachepages

# get gene info for snps that are not found in dbSNP
python2.5 $ERANGEPATH/getNovelSNPs.py $1 $3.nr_dbsnp_geneinfo.txt $3.nr.final.txt 

# make bed file for displaying the snps on UCSC genome browser
python2.5 $ERANGEPATH/makeSNPtrack.py $3.nr_snps.txt $3 $3.nr_snps.bed
